import axios from 'axios'
import { CursorPaginationParameters, PageNumberPaginationParameters, unique } from '.'
import { CursorPagination, PageNumberPagination, UUID } from '@/types'
import { Dataset, DatasetDetails, DatasetElementList, ElementDataset } from '@/types/dataset'

export interface DatasetCreate extends Omit<Dataset, 'id' | 'state'> {
  corpusId: UUID
}

export interface DatasetEdit extends Omit<Dataset, 'state'> {}

export interface DatasetParams extends PageNumberPaginationParameters {
  /**
   * ID of the corpus to retrieve datasets from.
   */
  corpusId: UUID
}

export const listCorpusDataset = unique(
  async ({ corpusId, ...params }: DatasetParams): Promise<PageNumberPagination<Dataset>> =>
    (await axios.get(`/corpus/${corpusId}/datasets/`, { params })).data
)

export const createDataset = unique(
  async ({ corpusId, ...data }: DatasetCreate): Promise<Dataset> =>
    (await axios.post(`/corpus/${corpusId}/datasets/`, data)).data
)

export const updateDataset = unique(
  async ({ id, ...data }: DatasetEdit): Promise<DatasetDetails> =>
    (await axios.patch(`/datasets/${id}/`, data)).data
)

export const retrieveDataset = unique(
  async (id: UUID): Promise<DatasetDetails> =>
    ((await axios.get(`/datasets/${id}/`)).data)
)

export const deleteDataset = unique(
  async (id: UUID) => await axios.delete(`/datasets/${id}/`)
)

// List datasets containing a specific element
export const listElementDatasets = unique(
  async (eltId: UUID, params: PageNumberPaginationParameters = {}): Promise<PageNumberPagination<ElementDataset>> =>
    (await axios.get(`/element/${eltId}/datasets/`, { params })).data
)

export const listDatasetElements = unique(
  async (datasetId: UUID, params: CursorPaginationParameters = {}): Promise<CursorPagination<DatasetElementList>> =>
    (await axios.get(`datasets/${datasetId}/elements/`, { params })).data
)

export const deleteDatasetElement = unique(
  async(datasetId: UUID, elementId: UUID, set: string) => await axios.delete(`datasets/${datasetId}/elements/${elementId}/`, { params: { set } })
)

export const cloneDataset = unique(
  async (datasetId: UUID): Promise<DatasetDetails> => (await axios.post(`datasets/${datasetId}/clone/`)).data
)
