import axios from 'axios'
import { UUID, PageNumberPagination } from '@/types'
import { OAuthCredentials, OAuthProvider } from '@/types/oauth'
import { unique, PageNumberPaginationParameters } from '.'

/**
 * List OAuth providers (GitLab, etc.).
 */
export const listProviders = unique(async (): Promise<OAuthProvider[]> => (await axios.get('/oauth/providers/')).data)

/**
 * List OAuth credentials (known accounts).
 */
export const listCredentials = unique(async (params: PageNumberPaginationParameters): Promise<PageNumberPagination<OAuthCredentials>> => (await axios.get('/oauth/credentials/', { params })).data)

/**
 * Delete a registered OAuth account.
 */
export const deleteCredentials = unique((id: UUID) => axios.delete(`/oauth/credentials/${id}/`))
