import axios from 'axios'
import { PageNumberPaginationParameters, unique } from '.'
import { PageNumberPagination, UUID } from '@/types'
import { Worker, WorkerType, WorkerVersion } from '@/types/worker'

export interface ListWorkersParameters extends PageNumberPaginationParameters {
  name?: string
  repository_id?: UUID
  type?: string
}

// List executable workers
export const listWorkers = unique(async (params: ListWorkersParameters): Promise<PageNumberPagination<Worker>> => (await axios.get('/workers/', { params })).data)

// List available worker types on instance
export const listWorkerTypes = unique(async (params: PageNumberPaginationParameters): Promise<PageNumberPagination<WorkerType>> => (await axios.get('/workers/types/', { params })).data)

export interface WorkerVersionListParameters extends PageNumberPaginationParameters {
  mode?: "simple" | "complete"
}

// List versions of a worker
export const listWorkerVersions = unique(async (workerId: UUID, params: WorkerVersionListParameters): Promise<PageNumberPagination<WorkerVersion>> => (await axios.get(`/workers/${workerId}/versions/`, { params })).data)

// Retrieve a worker version
export const retrieveWorkerVersion = unique(async (id: UUID): Promise<WorkerVersion> => (await axios.get(`/workers/versions/${id}/`)).data)

// Retrieve a worker
export const retrieveWorker = unique(async (id: UUID): Promise<Worker> => (await axios.get(`/workers/${id}/`)).data)
