import { defineStore } from 'pinia'
import { PageNumberPaginationParameters } from '@/api'
import { errorParser } from '@/helpers'
import { UUID, PageNumberPagination } from '@/types'
import { ElementDataset } from '@/types/dataset'
import { useNotificationStore } from '@/stores'
import axios from 'axios'
import * as api from '@/api'

interface State {
  /**
   * List of dataset that contains a specific element
   */
  elementDatasets: { [elementId: UUID]: ElementDataset[] }
}

export const useElementsStore = defineStore('elements', {
  state: (): State => ({
    elementDatasets: {}
  }),
  actions: {
    async listElementDatasets (eltId: UUID, params: PageNumberPaginationParameters = {}, url: string | null = null) {
      // Avoid listing datasets twice for an element
      if (url === null && (this.elementDatasets?.[eltId] || null) !== null) return
      // List datasets containing a specific element through all pages
      //let results, pagination
      try {
        const { results, ...pagination } = (
          url === null ?
          await api.listElementDatasets(eltId, params)
          : (await axios.get(url)).data as PageNumberPagination<ElementDataset>
        )
        // Progressively add results to the store
        this.elementDatasets[eltId] = [
          ...this.elementDatasets?.[eltId] || [],
          ...results
        ]
        // Follow pagination without awaiting, until we fetched all the data
        if (pagination.next !== null) this.listElementDatasets(eltId, params, pagination.next)
      } catch (err) {
        useNotificationStore().notify({ type: 'error', text: errorParser(err) })
        throw err
      }
    }
  }
})
